---------------------------------------------------------------------
--
-- ***************************************************************
-- Licensed Materials - Property of IBM
-- 5724-B96, 5724-V09
--
-- Copyright IBM Corp. 2005, 2008 All Rights Reserved
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with
-- IBM Corp
-- ***************************************************************
--
-- NAME
--    korgrantcustom.sql
-- FUNCTION
--   Grant access to all views used by the Monitoring Agent for Oracle.
-- NOTES
--   This file must be run while logged-in as SYS.
--   Catalog.sql must have been run before this file is run.
--
--
-- MODIFIED
---------------------------------------------------------------------
--   Maintainence Log
--   @AL01 - 11/06/95 Date Created
--   @AL02 - 11/28/95 Removed the drop role candler because of serious bug
--                    with multiple oracle servers on same host, previous
--		      users losing the permissions due to drop role command
--   @JG03 - 08/26/05 NEW, based upon korgrant.sql
--   @TF01 - 03/26/07 Added setting temporary tablespace
--   @TF02 - 03/26/07 Added role-based privledges
--   @TF03 - 04/04/07 Removed unlimited tablespace and execute any
--                    procedure. These permissions are not needed.
--   @TF04 - 08/07/14 Tuning sql, add some privilege to agent user.
--------------------------------------------------------------------------
-- Usage Instructions
-- Change directory to where this file is
--	cd <directory path  name of this file>
-- Bringup  sqlplus
--	sqlplus
--	login : sys
--	password : <enter  password for user SYS>
-- Create the user ID using the following command
--    create user <user ID> identified by <password>
--    where <user ID> is the user ID that the Oracle Agent will use
--          <password> is the desired password for the user ID
--    For example: create user tivoli identified by secret;
-- Create the role using the following command
--    create role <role_name>
--    For example: create role tivoli_role;
-- Run this script
--	   @korgrantcustom.sql <user ID> <tablespace name> <datafile> <tablespace size>
--                        <user role> <temporary tablespace name> <temporary datafile>
--                        <temporary tablespace size> <temporary dir>
--
-- (1)                 <user ID> = ID of the Oracle user.  Must be created
--                                 prior to running this sql file.
--
--                                 Example value: tivoli
--
-- (2)         <tablespace name> = Name of the tablespace to create and
--                                 associate with the user ID.
--
--                                 Example value: tivoliorts
--
-- (3)                <datafile> = The fully qualified path and
--                                 name of the datafile to use for the
--                                 tablespace.  If the path is omitted, the
--                                 datafile location is determined by either
--                                 Oracle or the operating system.
--
--                                 Example value: tivoliorts.dbf
--
-- (4)         <tablespace size> = Specifies how large the tablespace
--                                 must be.  Value needs to includes the
--                                 units.
--
--                                 Example value: 250M
--                                   (Creates a 250 megabyte tablespace.)
--
-- (5)               <user role> = Name of the role for the required Oracle
--                                 Agent permisisons to be granted to. Must
--                                 be created prior to running this sql
--                                 file.
--
--                                 Example value: tivoli_role
--
--(6)<temporary tablespace name> = Name of the temporary tablespace to create
--                                 and associate with the user ID.
--
--                                 Example value: tivoliortempts
--
-- (7)      <temporary datafile> = The fully qualified path and
--                                 name of the datafile to use for the
--                                 temporary tablespace.  If the path is omitted,
--                                 the datafile location is determined by either
--                                 Oracle or the operating system.
--
--                                 Example value: tivoliortempts.dbf
--
--(8)<temporary tablespace size> = Specifies how large the temporary tablespace
--                                 must be.  Value needs to includes the
--                                 units.
--
--                                 Example value: 250M
--                                   (Creates a 250 megabyte tablespace.)
--
-- (9)         <temporary dir>   = Name of the temporary directory where Oracle
--                                 Agent will write temporary files. Must
--                                 be created prior to running this sql
--                                 file.
--
--                                 Example value: /opt/IBM/ITM/tmp
--
-- Will output the log to the file korgrantcustom.log in the current directory
--
--------------------------------------------------------------------------

set echo on;
spool ./korgrantcustom_&2..log

-- Print diagnostic information
select '&1' username, 
       '&2' tablespace, 
       '&3' datafile , 
       '&4' tablespacesize , 
       '&5' role , 
       '&6' temp_tablespace , 
       '&7' temp_datafile , 
       '&8' temp_tablespace_size , 
       '&9' temp_dir, 
       to_char(sysdate, 'YYYY/MM/dd hh:mm:ss') time
  from dual;

select version, instance_name from v$instance;  
  
-- Create a directory called tivolior_temp and grant
-- read/write authority to it
create or replace directory tivolior_temp as '&9';
GRANT read,WRITE ON directory tivolior_temp TO &5;

-- Create and associate a tablespace with the user.
create tablespace &2 datafile '&3' size &4 autoextend off;
alter user &1 default tablespace &2 quota &4 on &2;
commit;

-- Create and associate a temporary tablespace with the user @TF01
create temporary tablespace &6 tempfile '&7' size &8;
alter user &1 temporary tablespace &6;
COMMIT;

-- Create view on X$KCCFE

CREATE OR REPLACE VIEW TIVOLI_FSTATUS_VIEW AS
SELECT FENUM FILE#,
       DECODE(FE.FETSN,
              0,
              DECODE(BITAND(FE.FESTA, 2), 0, 'SYSOFF', 'SYSTEM'),
              DECODE(BITAND(FE.FESTA, 18),
                     0,
                     'OFFLINE',
                     2,
                     'ONLINE',
                     'RECOVER')) STATUS,
                     DECODE(HC.KTFBHCCVAL, 0, HC.KTFBHCSZ, NULL) BLOCKS
  FROM X$KCCFE FE, SYS.X$KTFBHC HC
 WHERE FE.INST_ID = USERENV('INSTANCE') AND FE.FENUM = HC.KTFBHCAFNO;


-- Create view for korfiles

CREATE OR REPLACE VIEW SYS.TIVOLI_KORFILES_VIEW AS
SELECT E.UTSNAME,
       E.UFILENAME,
       E.FILE_ID FILERID,
       E.FILESTAT,
       DECODE(D.STATUS, 'NOT ACTIVE', 'INACTIVE') BKUPSTAT,
       D.TIME BKUPSTATM,
       C.TSNFREEB,
       C.LFREEBLK * 1024 MAXBYTES,
       (C.SUMBYTES / E.RAWFILESIZE) * 10000 PFREESPC,
       C.LFREEBLK,
       (C.LFREEBLK * 1024 / E.RAWFILESIZE) * 10000 LPCTFREE,
       E.RAWFILESIZE / 1024 FILESIZE,
       E.RAWFILESIZE,
       A.FILEXTENTS,
       B.PHYRDS PHYREAD,
       B.PHYWRTS PHYWRITE
  FROM V$BACKUP D,
       (SELECT T1.FILE# FILE_ID,
               T1.NAME UFILENAME,
               T3.NAME UTSNAME,
               FE.STATUS FILESTAT,
               FE.BLOCKS * T3.BLOCKSIZE RAWFILESIZE
          FROM V$DBFILE                T1,
               SYS.FILE$               T2,
               SYS.TS$                 T3,
               SYS.TIVOLI_FSTATUS_VIEW FE
         WHERE T1.FILE# = T2.FILE#
           AND T2.TS# = T3.TS#
           AND T3.ONLINE$ <> 3
           AND T1.FILE# = FE.FILE#) E,
(SELECT FILE# FILERID, sum(extents) FILEXTENTS
          FROM seg$
          group by file#) A,
       V$FILESTAT B,
       (SELECT FILE_ID FILERID,
               SUM(BLOCKS) TSNFREEB,
               MAX(BYTES / 1024) LFREEBLK,
               SUM(BYTES) SUMBYTES
          FROM SYS.DBA_FREE_SPACE
         GROUP BY FILE_ID) C
 WHERE A.FILERID(+) = B.FILE#
   AND C.FILERID(+) = B.FILE#
   AND D.FILE#(+) = B.FILE#
   AND E.FILE_ID = B.FILE#
 UNION ALL
SELECT T1.TABLESPACE_NAME UTSNAME,
       T1.FILE_NAME UFILENAME,
       T1.FILE_ID,
       T2.STATUS FILESTAT,
       'INACTIVE' BKUPSTAT,
       NULL BKUPSTATM,
       T1.BLOCKS - NVL(T3.USED_BLOCKS, 0) TSNFREEB,
       NULL MAXBYTES,
       (T1.BLOCKS - NVL(T3.USED_BLOCKS, 0)) / T1.BLOCKS*10000 PFREESPC,
       NULL LFREEBLK,
       NULL LPCTFREE,
       T1.BYTES / 1024 FILESIZE,
       T1.BYTES RAWFILESIZE,
       NVL(T3.TOTAL_EXTENTS, 0) FILEXTENTS,
       T5.PHYRDS PHYREAD,
       T5.PHYWRTS PHYWRITE
  FROM DBA_TEMP_FILES T1,
       V$TEMPFILE T2,
       (SELECT TABLESPACE_NAME,
               SEGMENT_FILE + 1 FILE_ID,
               EXTENT_SIZE,
               TOTAL_EXTENTS,
               TOTAL_BLOCKS,
               USED_EXTENTS,
               USED_BLOCKS
          FROM V$SORT_SEGMENT) T3,
       V$TEMPSTAT T5
 WHERE T1.FILE_ID = T2.FILE#
   AND T1.FILE_ID = T5.FILE#
   AND T3.TABLESPACE_NAME(+) = T1.TABLESPACE_NAME
   AND T3.FILE_ID(+) = T1.FILE_ID
 ORDER BY 3;

-- Create view for korfiles but FILEXTENTS is disabled by default to avoid performance issue

CREATE OR REPLACE VIEW SYS.TIVOLI_KORFILES_VIEW_NOFE AS
SELECT E.UTSNAME,
       E.UFILENAME,
       E.FILE_ID FILERID,
       E.FILESTAT,
       DECODE(D.STATUS, 'NOT ACTIVE', 'INACTIVE') BKUPSTAT,
       D.TIME BKUPSTATM,
       C.TSNFREEB,
       C.LFREEBLK * 1024 MAXBYTES,
       (C.SUMBYTES / E.RAWFILESIZE) * 10000 PFREESPC,
       C.LFREEBLK,
       (C.LFREEBLK * 1024 / E.RAWFILESIZE) * 10000 LPCTFREE,
       E.RAWFILESIZE / 1024 FILESIZE,
       E.RAWFILESIZE,
       -1 FILEXTENTS,
       B.PHYRDS PHYREAD,
       B.PHYWRTS PHYWRITE
  FROM V$BACKUP D,
       (SELECT T1.FILE# FILE_ID,
               T1.NAME UFILENAME,
               T3.NAME UTSNAME,
               FE.STATUS FILESTAT,
               FE.BLOCKS * T3.BLOCKSIZE RAWFILESIZE
          FROM V$DBFILE                T1,
               SYS.FILE$               T2,
               SYS.TS$                 T3,
               SYS.TIVOLI_FSTATUS_VIEW FE
         WHERE T1.FILE# = T2.FILE#
           AND T2.TS# = T3.TS#
           AND T3.ONLINE$ <> 3
           AND T1.FILE# = FE.FILE#) E,
       V$FILESTAT B,
       (SELECT FILE_ID FILERID,
               SUM(BLOCKS) TSNFREEB,
               MAX(BYTES / 1024) LFREEBLK,
               SUM(BYTES) SUMBYTES
          FROM SYS.DBA_FREE_SPACE
         GROUP BY FILE_ID) C
 WHERE C.FILERID(+) = B.FILE#
   AND D.FILE#(+) = B.FILE#
   AND E.FILE_ID = B.FILE#
 UNION ALL
SELECT T1.TABLESPACE_NAME UTSNAME,
       T1.FILE_NAME UFILENAME,
       T1.FILE_ID,
       T2.STATUS FILESTAT,
       'INACTIVE' BKUPSTAT,
       NULL BKUPSTATM,
       T1.BLOCKS - NVL(T3.USED_BLOCKS, 0) TSNFREEB,
       NULL MAXBYTES,
       (T1.BLOCKS - NVL(T3.USED_BLOCKS, 0)) / T1.BLOCKS *10000 PFREESPC,
       NULL LFREEBLK,
       NULL LPCTFREE,
       T1.BYTES / 1024 FILESIZE,
       T1.BYTES RAWFILESIZE,
       -1 FILEXTENTS,
       T5.PHYRDS PHYREAD,
       T5.PHYWRTS PHYWRITE
  FROM DBA_TEMP_FILES T1,
       V$TEMPFILE T2,
       (SELECT TABLESPACE_NAME,
               SEGMENT_FILE + 1 FILE_ID,
               EXTENT_SIZE,
               TOTAL_EXTENTS,
               TOTAL_BLOCKS,
               USED_EXTENTS,
               USED_BLOCKS
          FROM V$SORT_SEGMENT) T3,
       V$TEMPSTAT T5
 WHERE T1.FILE_ID = T2.FILE#
   AND T1.FILE_ID = T5.FILE#
   AND T3.TABLESPACE_NAME(+) = T1.TABLESPACE_NAME
   AND T3.FILE_ID(+) = T1.FILE_ID
 ORDER BY 3;

-- Serious bug @AL02 commented out
-- drop role candler;
-- create role &1;

------------------------------------------------
-- V$ grants first
------------------------------------------------
grant select on   V_$ACCESS         to &5;
grant select on   V_$ARCHIVE_DEST   to &5;
grant select on   V_$BACKUP         to &5;
grant select on   V_$BGPROCESS      to &5;
grant select on   V_$BUFFER_POOL_STATISTICS to &5;
grant select on   V_$DATABASE       to &5;
grant select on   V_$DATAFILE       to &5;
grant select on   V_$DBFILE         to &5;
grant select on   V_$DISPATCHER     to &5;
grant select on   V_$FILESTAT       to &5;
grant select on   V_$HS_AGENT       to &5;
grant select on   V_$HS_SESSION     to &5;
grant select on   V_$INSTANCE       to &5;
grant select on   V_$LATCH          to &5;
grant select on   V_$LIBRARYCACHE   to &5;
grant select on   V_$LICENSE        to &5;
grant select on   V_$LOCK           to &5;
grant select on   V_$LOCK_ACTIVITY  to &5;
grant select on   V_$LOCKED_OBJECT  to &5;
grant select on   V_$LOG            to &5;

prompt If Oracle Database 10g or above 10g is being used,
prompt please ignore the error on granting V_$MTS of following.

grant select on   V_$MTS            to &5;
grant select on   V_$OPEN_CURSOR    to &5;
grant select on   V_$OPTION         to &5;
grant select on   V_$PARAMETER      to &5;
grant select on   V_$PGASTAT        to &5;
grant select on   V_$PROCESS        to &5;
grant select on   V_$PX_PROCESS_SYSSTAT to &5;
grant select on   V_$QUEUE          to &5;
grant select on   V_$ROLLNAME       to &5;
grant select on   V_$ROLLSTAT       to &5;
grant select on   V_$ROWCACHE       to &5;
grant select on   V_$SESSION        to &5;
grant select on   V_$SESSION_WAIT   to &5;
grant select on   V_$SESS_IO        to &5;
grant select on   V_$SGA            to &5;
grant select on   V_$SGA_DYNAMIC_COMPONENTS to &5;
grant select on   V_$SGASTAT        to &5;
grant select on   V_$SHARED_SERVER  to &5;
grant select on   V_$SHARED_SERVER_MONITOR  to &5;
grant select on   V_$SORT_USAGE     to &5;
grant select on   V_$SQLTEXT        to &5;
grant select on   V_$SQLTEXT_WITH_NEWLINES  to &5;
grant select on   V_$SYSSTAT        to &5;
grant select on   V_$SYSTEM_EVENT   to &5;
grant select on   V_$TABLESPACE     to &5;
grant select on   V_$TEMPFILE       to &5;
grant select on   V_$TEMP_EXTENT_MAP to &5;
grant select on   V_$THREAD         to &5;
grant select on   V_$TRANSACTION    to &5;
grant select on   V_$UNDOSTAT       to &5;
grant select on   V_$VERSION        to &5;
grant select on   V_$WAITSTAT       to &5;
grant select on   v_$datafile_header to &5;
grant select on   v_$enqueue_lock to &5;
grant select on   v_$sort_segment to &5;

------------------------------------------------
-- DBA_ required tables
------------------------------------------------
grant select on   SYS.DBA_CLUSTERS     to &5;
grant select on   SYS.DBA_DATA_FILES   to &5;
grant select on   SYS.DBA_DIRECTORIES  to &5;
grant select on   SYS.DBA_EXTENTS      to &5;
grant select on   SYS.DBA_FREE_SPACE   to &5;
grant select on   SYS.DBA_INDEXES      to &5;
grant select on   SYS.DBA_JOBS         to &5;
grant select on   SYS.DBA_LMT_USED_EXTENTS to &5;
grant select on   SYS.DBA_DMT_USED_EXTENTS to &5;
grant select on   SYS.DBA_QUEUES       to &5;
grant select on   SYS.DBA_QUEUE_SCHEDULES  to &5;
grant select on   SYS.DBA_SEGMENTS     to &5;
grant select on   SYS.DBA_TABLES       to &5;
grant select on   SYS.DBA_TABLESPACES  to &5;
grant select on   SYS.DBA_TEMP_FILES   to &5;
grant select on   SYS.DBA_VIEWS        to &5;
grant select on   SYS.DBA_SYNONYMS     to &5;
grant select on   SYS.fet$             to &5;
grant select on   SYS.file$            to &5;
grant select on   SYS.OBJ$             to &5;
grant select on   SYS.sys_dba_segs     to &5;
grant select on   SYS.SEG$             to &5;
grant select on   SYS.ts$              to &5;
grant select on   SYS.TIVOLI_FSTATUS_VIEW  to &5;
grant select on   SYS.TIVOLI_KORFILES_VIEW  to &5;
grant select on   SYS.TIVOLI_KORFILES_VIEW_NOFE  to &5;
grant select on   SYS.UNDO$            to &5;
grant select on   SYS.uet$             to &5;
grant select on   SYS.GV_$AQ           to &5;

------------------------------------------------
-- Grant create session to the user
------------------------------------------------
grant connect               to &1;
grant create session        to &1;

grant restricted session    to &1;
------------------------------------------------
-- Grants needed to collect attributes that
-- use PL/SQL:
--   - Free Space Deficit   (Tablespace Segments)
--   - Table Chained Rows   (Table Summary)
--   - Cluster Chained Rows (Cluster Summary)
--   - Index Percent Used   (Index Summary)
------------------------------------------------
-- grant create PROCEDURE 		to &5;
grant create TABLE 			to &5;
-- @TF03 - removed unneeded permissions
-- grant execute any PROCEDURE 		to &1;
-- grant unlimited tablespace 		to &1;

------------------------------------------------
-- Grants needed to collect attributes that
-- use analyze cluster|table or
--     validate index:
--   - Table Chained Rows    (Table Summary)
--   - Cluster Chained Rows  (Cluster Summary)
--   - Index Percent Deleted (Index Summary)
------------------------------------------------
GRANT analyze ANY                TO &5;


------------------------------------------------
-- Grant role candler to the user
------------------------------------------------
-- @AL02 grant candler              to &1;

GRANT &5 TO &1;

spool off
set echo off;
------------------------------------------------
